@@ -20,11 +20,11 @@
 #define KDIRLISTERTEST_H
 
 #include <QSignalSpy>
-#include <QtCore/QObject>
+#include <QObject>
 #include <qtemporarydir.h>
-#include <QtCore/QDate>
+#include <QDate>
 #include <kdirlister.h>
-#include <QtCore/QEventLoop>
+#include <QEventLoop>
 
 Q_DECLARE_METATYPE(KFileItemList)
 
@@ -77,7 +77,7 @@ public:
     QSignalSpy spyRedirection;
     QSignalSpy spyItemsDeleted;
 protected:
-    virtual void handleError(KIO::Job *job);
+    void handleError(KIO::Job *job) override;
 };
 
 class KDirListerTest : public QObject
@@ -88,12 +88,15 @@ private Q_SLOTS:
     void cleanup();
     void testOpenUrl();
     void testOpenUrlFromCache();
+    void testNewItem();
     void testNewItems();
+    void benchFindByUrl();
     void testNewItemByCopy();
     void testNewItemsInSymlink();
     void testRefreshItems();
     void testRefreshRootItem();
     void testDeleteItem();
+    void testDeleteItems();
     void testRenameItem();
     void testRenameAndOverwrite();
     void testConcurrentListing();
@@ -112,10 +115,10 @@ private Q_SLOTS:
     void testRemoveWatchedDirectory();
     void testDirPermissionChange();
     void testCopyAfterListingAndMove(); // #353195
+    void testRenameDirectory(); // #401552
     void testDeleteCurrentDir(); // must be last!
 
 protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtestlib
-    void exitLoop();
     void slotNewItems(const KFileItemList &);
     void slotNewItems2(const KFileItemList &);
     void slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &);
@@ -126,13 +129,11 @@ Q_SIGNALS:
     void refreshItemsReceived();
 
 private:
-    void enterLoop(int exitCount = 1);
     int fileCount() const;
     QString path() const
     {
         return m_tempDir.path() + '/';
     }
-    bool waitForRefreshedItems();
     void createSimpleFile(const QString &fileName);
     void fillDirLister2(MyDirLister &lister, const QString &path);
     void waitUntilMTimeChange(const QString &path);
