@@ -34,9 +34,7 @@
 #include <QMimeData>
 #include <QTimer>
 #include <QFile>
-#include <QColor>
 #include <QAction>
-#include <QUrlQuery>
 #include <qmimedatabase.h>
 
 #include <kfileitem.h>
@@ -109,7 +107,7 @@ namespace {
     static QUrl createTimelineUrl(const QUrl &url)
     {
         // based on dolphin urls
-        const QString timelinePrefix = QStringLiteral("timeline:") + QLatin1Char('/');
+        const QString timelinePrefix = QLatin1String("timeline:") + QLatin1Char('/');
         QUrl timelineUrl;
 
         const QString path = url.toDisplayString(QUrl::PreferLocalFile);
@@ -250,6 +248,19 @@ private:
     bool isBalooUrl(const QUrl &url) const;
 };
 
+KBookmark KFilePlacesModel::bookmarkForUrl(const QUrl &searchUrl) const
+{
+    KBookmarkGroup root = d->bookmarkManager->root();
+    KBookmark current = root.first();
+    while (!current.isNull()) {
+        if (current.url() == searchUrl) {
+            return current;
+        }
+        current = root.next(current);
+    }
+    return KBookmark();
+}
+
 KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QObject *parent)
     : QAbstractItemModel(parent), d(new Private(this))
 {
@@ -302,19 +313,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", "Network"),
                                               QUrl(QStringLiteral("remote:/")), QStringLiteral("folder-network"));
-#if defined(_WIN32_WCE)
-        // adding drives
-        const QString driveIcon = QStringLiteral("drive-harddisk");
-        foreach (const QFileInfo &info, QDir::drives()) {
-            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                                  info.absoluteFilePath(), info.absoluteFilePath(),
-                                                  QUrl::fromLocalFile(info.absoluteFilePath()), driveIcon);
-        }
-#elif !defined(Q_OS_WIN)
-        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Root"), I18N_NOOP2("KFile System Bookmarks", "Root"),
-                                              QUrl::fromLocalFile(QStringLiteral("/")), QStringLiteral("folder-root"));
-#endif
+
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", "Trash"),
                                               QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"));
@@ -331,16 +330,42 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         d->bookmarkManager->saveAs(file);
     }
 
+    // Add a Recently Used entry if available (it comes from kio-extras)
+    if (qEnvironmentVariableIsSet("KDE_FULL_SESSION") && KProtocolInfo::isKnownProtocol(QStringLiteral("recentlyused")) &&
+             root.metaDataItem(QStringLiteral("withRecentlyUsed")) != QLatin1String("true")) {
+
+        root.setMetaDataItem(QStringLiteral("withRecentlyUsed"), QStringLiteral("true"));
+
+        KBookmark recentFilesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                            QStringLiteral("Recent Files"), I18N_NOOP2("KFile System Bookmarks", "Recent Files"),
+                                            QUrl(QStringLiteral("recentlyused:/files")), QStringLiteral("document-open-recent"));
+
+        KBookmark recentDirectoriesBookmark = KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                            QStringLiteral("Recent Locations"), I18N_NOOP2("KFile System Bookmarks", "Recent Locations"),
+                                            QUrl(QStringLiteral("recentlyused:/locations")), QStringLiteral("folder-open-recent"));
+
+        setDefaultMetadataItemForGroup(RecentlySavedType);
+
+        // Move The recently used bookmarks below the trash, making it the first element in the Recent group
+        KBookmark trashBookmark = bookmarkForUrl(QUrl("trash:/"));
+        if (!trashBookmark.isNull()) {
+            root.moveBookmark(recentFilesBookmark, trashBookmark);
+            root.moveBookmark(recentDirectoriesBookmark, recentFilesBookmark);
+        }
+
+        d->bookmarkManager->save();
+    }
+
     // if baloo is enabled, add new urls even if the bookmark file is not empty
     if (d->fileIndexingEnabled &&
         root.metaDataItem(QStringLiteral("withBaloo")) != QLatin1String("true")) {
 
         root.setMetaDataItem(QStringLiteral("withBaloo"), QStringLiteral("true"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Today"), I18N_NOOP2("KFile System Bookmarks", "Today"),
+                                              QStringLiteral("Modified Today"), I18N_NOOP2("KFile System Bookmarks", "Today"),
                                               QUrl(QStringLiteral("timeline:/today")),  QStringLiteral("go-jump-today"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Yesterday"), I18N_NOOP2("KFile System Bookmarks", "Yesterday"),
+                                              QStringLiteral("Modified Yesterday"), I18N_NOOP2("KFile System Bookmarks", "Yesterday"),
                                               QUrl(QStringLiteral("timeline:/yesterday")),  QStringLiteral("view-calendar-day"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
@@ -349,7 +374,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
                                               QStringLiteral("Images"), I18N_NOOP2("KFile System Bookmarks", "Images"),
                                               QUrl(QStringLiteral("search:/images")),  QStringLiteral("folder-images"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Audio Files"), I18N_NOOP2("KFile System Bookmarks", "Audio Files"),
+                                              QStringLiteral("Audio"), I18N_NOOP2("KFile System Bookmarks", "Audio"),
                                               QUrl(QStringLiteral("search:/audio")),  QStringLiteral("folder-sound"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Videos"), I18N_NOOP2("KFile System Bookmarks", "Videos"),
@@ -425,7 +450,7 @@ bool KFilePlacesModel::isHidden(const QModelIndex &index) const
 bool KFilePlacesModel::isGroupHidden(const GroupType type) const
 {
     const QString hidden = d->bookmarkManager->root().metaDataItem(stateNameForGroupType(type));
-    return hidden == QStringLiteral("true") ? true : false;
+    return hidden == QLatin1String("true") ? true : false;
 }
 
 bool KFilePlacesModel::isGroupHidden(const QModelIndex &index) const
