@@ -21,28 +21,30 @@
 #include "kfileplacesview.h"
 #include "kfileplacesview_p.h"
 
-#include <QtCore/QTimeLine>
-#include <QtCore/QTimer>
+#include <QDir>
+#include <QTimeLine>
+#include <QTimer>
 #include <QPainter>
 #include <QAbstractItemDelegate>
 #include <QKeyEvent>
 #include <QApplication>
 #include <QMenu>
+#include <QPointer>
 #include <QScrollBar>
 
-#include <QDebug>
-
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <kdirnotify.h>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
+#include <kmountpoint.h>
+#include <kpropertiesdialog.h>
 #include <kio/emptytrashjob.h>
+#include <kio/filesystemfreespacejob.h>
 #include <kio/jobuidelegate.h>
 #include <kjob.h>
 #include <kjobwidgets.h>
 #include <kcapacitybar.h>
-#include <kdiskfreespaceinfo.h>
 #include <solid/storageaccess.h>
 #include <solid/storagedrive.h>
 #include <solid/storagevolume.h>
@@ -55,17 +57,25 @@
 #define LATERAL_MARGIN 4
 #define CAPACITYBAR_HEIGHT 6
 
+struct PlaceFreeSpaceInfo
+{
+    QDateTime lastUpdated;
+    KIO::filesize_t used = 0;
+    KIO::filesize_t size = 0;
+    QPointer<KIO::FileSystemFreeSpaceJob> job;
+};
+
 class KFilePlacesViewDelegate : public QAbstractItemDelegate
 {
     Q_OBJECT
 public:
-    KFilePlacesViewDelegate(KFilePlacesView *parent);
-    virtual ~KFilePlacesViewDelegate();
-    virtual QSize sizeHint(const QStyleOptionViewItem &option,
-                           const QModelIndex &index) const Q_DECL_OVERRIDE;
-    virtual void paint(QPainter *painter,
-                       const QStyleOptionViewItem &option,
-                       const QModelIndex &index) const Q_DECL_OVERRIDE;
+    explicit KFilePlacesViewDelegate(KFilePlacesView *parent);
+    ~KFilePlacesViewDelegate() override;
+    QSize sizeHint(const QStyleOptionViewItem &option,
+                   const QModelIndex &index) const override;
+    void paint(QPainter *painter,
+               const QStyleOptionViewItem &option,
+               const QModelIndex &index) const override;
 
     int iconSize() const;
     void setIconSize(int newSize);
@@ -73,6 +83,7 @@ public:
     void addAppearingItem(const QModelIndex &index);
     void setAppearingItemProgress(qreal value);
     void addDisappearingItem(const QModelIndex &index);
+    void addDisappearingItemGroup(const QModelIndex &index);
     void setDisappearingItemProgress(qreal value);
 
     void setShowHoverIndication(bool show);
@@ -90,6 +101,8 @@ public:
 
     int sectionHeaderHeight() const;
 
+    void clearFreeSpaceInfo();
+
 private:
     QString groupNameFromIndex(const QModelIndex &index) const;
     QModelIndex previousVisibleIndex(const QModelIndex &index) const;
@@ -118,6 +131,8 @@ private:
 
     QMap<QPersistentModelIndex, QTimeLine *> m_timeLineMap;
     QMap<QTimeLine *, QPersistentModelIndex> m_timeLineInverseMap;
+
+    mutable QMap<QPersistentModelIndex, PlaceFreeSpaceInfo> m_freeSpaceInfo;
 };
 
 KFilePlacesViewDelegate::KFilePlacesViewDelegate(KFilePlacesView *parent) :
@@ -169,10 +184,9 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
             drawSectionHeader(painter, opt, index);
         }
 
+        // Move the target rect to the actual item rect
         const int headerHeight = sectionHeaderHeight();
-        const int headerSpace =  (headerHeight / 2) + qMax(2, m_view->spacing());
-        painter->translate(0, headerSpace);
-        opt.rect.translate(0, headerSpace);
+        opt.rect.translate(0, headerHeight);
         opt.rect.setHeight(opt.rect.height() - headerHeight);
     }
 
@@ -192,59 +206,80 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter);
     const KFilePlacesModel *placesModel = static_cast<const KFilePlacesModel *>(index.model());
 
-    bool isLTR = option.direction == Qt::LeftToRight;
+    bool isLTR = opt.direction == Qt::LeftToRight;
 
     QIcon icon = index.model()->data(index, Qt::DecorationRole).value<QIcon>();
-    QPixmap pm = icon.pixmap(m_iconSize, m_iconSize, (option.state & QStyle::State_Selected) && (option.state & QStyle::State_Active) ? QIcon::Selected : QIcon::Normal);
-    QPoint point(isLTR ? option.rect.left() + LATERAL_MARGIN
-                 : option.rect.right() - LATERAL_MARGIN - m_iconSize, option.rect.top() + (option.rect.height() - m_iconSize) / 2);
+    QPixmap pm = icon.pixmap(m_iconSize, m_iconSize, (opt.state & QStyle::State_Selected) && (opt.state & QStyle::State_Active) ? QIcon::Selected : QIcon::Normal);
+    QPoint point(isLTR ? opt.rect.left() + LATERAL_MARGIN
+                 : opt.rect.right() - LATERAL_MARGIN - m_iconSize, opt.rect.top() + (opt.rect.height() - m_iconSize) / 2);
     painter->drawPixmap(point, pm);
 
-    if (option.state & QStyle::State_Selected) {
+    if (opt.state & QStyle::State_Selected) {
         QPalette::ColorGroup cg = QPalette::Active;
-        if (!(option.state & QStyle::State_Enabled)) {
+        if (!(opt.state & QStyle::State_Enabled)) {
             cg = QPalette::Disabled;
-        } else if (!(option.state & QStyle::State_Active)) {
+        } else if (!(opt.state & QStyle::State_Active)) {
             cg = QPalette::Inactive;
         }
-        painter->setPen(option.palette.color(cg, QPalette::HighlightedText));
+        painter->setPen(opt.palette.color(cg, QPalette::HighlightedText));
     }
 
     QRect rectText;
 
-    const QUrl url = placesModel->url(index);
     bool drawCapacityBar = false;
-    if (url.isLocalFile()) {
-        const QString mountPointPath = placesModel->url(index).toLocalFile();
-        const KDiskFreeSpaceInfo info = KDiskFreeSpaceInfo::freeSpaceInfo(mountPointPath);
-        drawCapacityBar = info.size() != 0 &&
-                          placesModel->data(index, KFilePlacesModel::CapacityBarRecommendedRole).toBool();
+    if (placesModel->data(index, KFilePlacesModel::CapacityBarRecommendedRole).toBool()) {
+        const QUrl url = placesModel->url(index);
+        if (contentsOpacity(index) > 0) {
+            QPersistentModelIndex persistentIndex(index);
+            PlaceFreeSpaceInfo &info = m_freeSpaceInfo[persistentIndex];
 
-        if (drawCapacityBar && contentsOpacity(index) > 0) {
-            painter->save();
-            painter->setOpacity(painter->opacity() * contentsOpacity(index));
+            drawCapacityBar = info.size > 0;
+            if (drawCapacityBar) {
+                painter->save();
+                painter->setOpacity(painter->opacity() * contentsOpacity(index));
 
-            int height = option.fontMetrics.height() + CAPACITYBAR_HEIGHT;
-            rectText = QRect(isLTR ? m_iconSize + LATERAL_MARGIN * 2 + option.rect.left()
-                             : 0, option.rect.top() + (option.rect.height() / 2 - height / 2), option.rect.width() - m_iconSize - LATERAL_MARGIN * 2, option.fontMetrics.height());
-            painter->drawText(rectText, Qt::AlignLeft | Qt::AlignTop, option.fontMetrics.elidedText(index.model()->data(index).toString(), Qt::ElideRight, rectText.width()));
-            QRect capacityRect(isLTR ? rectText.x() : LATERAL_MARGIN, rectText.bottom() - 1, rectText.width() - LATERAL_MARGIN, CAPACITYBAR_HEIGHT);
-            KCapacityBar capacityBar(KCapacityBar::DrawTextInline);
-            capacityBar.setValue((info.used() * 100) / info.size());
-            capacityBar.drawCapacityBar(painter, capacityRect);
+                int height = opt.fontMetrics.height() + CAPACITYBAR_HEIGHT;
+                rectText = QRect(isLTR ? m_iconSize + LATERAL_MARGIN * 2 + opt.rect.left()
+                                 : 0, opt.rect.top() + (opt.rect.height() / 2 - height / 2), opt.rect.width() - m_iconSize - LATERAL_MARGIN * 2, opt.fontMetrics.height());
+                painter->drawText(rectText, Qt::AlignLeft | Qt::AlignTop, opt.fontMetrics.elidedText(index.model()->data(index).toString(), Qt::ElideRight, rectText.width()));
+                QRect capacityRect(isLTR ? rectText.x() : LATERAL_MARGIN, rectText.bottom() - 1, rectText.width() - LATERAL_MARGIN, CAPACITYBAR_HEIGHT);
+                KCapacityBar capacityBar(KCapacityBar::DrawTextInline);
+                capacityBar.setValue((info.used * 100) / info.size);
+                capacityBar.drawCapacityBar(painter, capacityRect);
 
-            painter->restore();
+                painter->restore();
 
-            painter->save();
-            painter->setOpacity(painter->opacity() * (1 - contentsOpacity(index)));
+                painter->save();
+                painter->setOpacity(painter->opacity() * (1 - contentsOpacity(index)));
+            }
+
+            if (!info.job &&
+                    (!info.lastUpdated.isValid() || info.lastUpdated.secsTo(QDateTime::currentDateTimeUtc()) > 60)) {
+                info.job = KIO::fileSystemFreeSpace(url);
+                connect(info.job, &KIO::FileSystemFreeSpaceJob::result, this, [this, persistentIndex](KIO::Job *job, KIO::filesize_t size, KIO::filesize_t available) {
+                    PlaceFreeSpaceInfo &info = m_freeSpaceInfo[persistentIndex];
+
+                    // even if we receive an error we want to refresh lastUpdated to avoid repeatedly querying in this case
+                    info.lastUpdated = QDateTime::currentDateTimeUtc();
+
+                    if (job->error()) {
+                        return;
+                    }
+
+                    info.size = size;
+                    info.used = size - available;
+
+                    // FIXME scheduleDelayedItemsLayout but we're in the delegate here, not the view
+                });
+            }
         }
     }
 
-    rectText = QRect(isLTR ? m_iconSize + LATERAL_MARGIN * 2 + option.rect.left()
-                     : 0, option.rect.top(), option.rect.width() - m_iconSize - LATERAL_MARGIN * 2, option.rect.height());
-    painter->drawText(rectText, Qt::AlignLeft | Qt::AlignVCenter, option.fontMetrics.elidedText(index.model()->data(index).toString(), Qt::ElideRight, rectText.width()));
+    rectText = QRect(isLTR ? m_iconSize + LATERAL_MARGIN * 2 + opt.rect.left()
+                     : 0, opt.rect.top(), opt.rect.width() - m_iconSize - LATERAL_MARGIN * 2, opt.rect.height());
+    painter->drawText(rectText, Qt::AlignLeft | Qt::AlignVCenter, opt.fontMetrics.elidedText(index.model()->data(index).toString(), Qt::ElideRight, rectText.width()));
 
-    if (drawCapacityBar && contentsOpacity(index) > 0) {
+    if (drawCapacityBar) {
         painter->restore();
     }
 
@@ -290,6 +325,16 @@ void KFilePlacesViewDelegate::addDisappearingItem(const QModelIndex &index)
     m_disappearingItems << index;
 }
 
+void KFilePlacesViewDelegate::addDisappearingItemGroup(const QModelIndex &index)
+{
+    const KFilePlacesModel *placesModel = static_cast<const KFilePlacesModel *>(index.model());
+    const QModelIndexList indexesGroup = placesModel->groupIndexes(placesModel->groupType(index));
+
+    m_disappearingItems.reserve(m_disappearingItems.count() + indexesGroup.count());
+    std::transform(indexesGroup.begin(), indexesGroup.end(), std::back_inserter(m_disappearingItems),
+                   [](const QModelIndex &idx){ return QPersistentModelIndex(idx); });
+}
+
 void KFilePlacesViewDelegate::setDisappearingItemProgress(qreal value)
 {
     value = 1.0 - value;
@@ -371,6 +416,11 @@ void KFilePlacesViewDelegate::startDrag()
     m_dragStarted = true;
 }
 
+void KFilePlacesViewDelegate::clearFreeSpaceInfo()
+{
+    m_freeSpaceInfo.clear();
+}
+
 QString KFilePlacesViewDelegate::groupNameFromIndex(const QModelIndex &index) const
 {
     if (index.isValid()) {
@@ -416,15 +466,21 @@ bool KFilePlacesViewDelegate::indexIsSectionHeader(const QModelIndex &index) con
 
 void KFilePlacesViewDelegate::drawSectionHeader(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-    const QString category = index.data(KFilePlacesModel::GroupRole).toString();
+    const KFilePlacesModel *placesModel = static_cast<const KFilePlacesModel *>(index.model());
+
+    const QString groupLabel = index.data(KFilePlacesModel::GroupRole).toString();
+    const QString category = placesModel->isGroupHidden(index) ? i18n("%1 (hidden)", groupLabel) : groupLabel;
+
     QRect textRect(option.rect);
     textRect.setLeft(textRect.left() + 3);
-    textRect.setY(textRect.y() + qMax(2, m_view->spacing()));
+    /* Take spacing into account:
+       The spacing to the previous section compensates for the spacing to the first item.*/
+    textRect.setY(textRect.y() /* + qMax(2, m_view->spacing()) - qMax(2, m_view->spacing())*/);
     textRect.setHeight(sectionHeaderHeight());
 
     painter->save();
 
-    // based on dolphoin colors
+    // based on dolphin colors
     const QColor c1 = textColor(option);
     const QColor c2 = baseColor(option);
     QColor penColor = mixedColor(c1, c2, 60);
@@ -458,15 +514,18 @@ QColor KFilePlacesViewDelegate::mixedColor(const QColor& c1, const QColor& c2, i
 
 int KFilePlacesViewDelegate::sectionHeaderHeight() const
 {
-    return QApplication::fontMetrics().height() +
-            (qMax(2, m_view->spacing()) * 2);
+    // Account for the spacing between header and item
+    return QApplication::fontMetrics().height() + qMax(2, m_view->spacing());
 }
 
 
 class Q_DECL_HIDDEN KFilePlacesView::Private
 {
 public:
-    Private(KFilePlacesView *parent) : q(parent), watcher(new KFilePlacesEventWatcher(q)) { }
+    explicit Private(KFilePlacesView *parent)
+        : q(parent)
+        , watcher(new KFilePlacesEventWatcher(q))
+    {}
 
     enum FadeType {
         FadeIn = 0,
@@ -481,7 +540,7 @@ public:
     bool smoothItemResizing;
     bool dropOnPlace;
     bool dragging;
-    Solid::StorageAccess *lastClickedStorage;
+    Solid::StorageAccess *lastClickedStorage = nullptr;
     QPersistentModelIndex lastClickedIndex;
 
     QRect dropRect;
@@ -489,12 +548,18 @@ public:
     void setCurrentIndex(const QModelIndex &index);
     void adaptItemSize();
     void updateHiddenRows();
+    void clearFreeSpaceInfos();
     bool insertAbove(const QRect &itemRect, const QPoint &pos) const;
     bool insertBelow(const QRect &itemRect, const QPoint &pos) const;
     int insertIndicatorHeight(int itemHeight) const;
     void fadeCapacityBar(const QModelIndex &index, FadeType fadeType);
     int sectionsCount() const;
 
+    void addDisappearingItem(KFilePlacesViewDelegate *delegate, const QModelIndex &index);
+
+    void triggerItemAppearingAnimation();
+    void triggerItemDisappearingAnimation();
+
     void _k_placeClicked(const QModelIndex &index);
     void _k_placeEntered(const QModelIndex &index);
     void _k_placeLeft(const QModelIndex &index);
@@ -513,7 +578,7 @@ public:
     QTimeLine itemDisappearTimeline;
 
     KFilePlacesEventWatcher *const watcher;
-    KFilePlacesViewDelegate *delegate;
+    KFilePlacesViewDelegate *delegate = nullptr;
     QTimer pollDevices;
     int pollingRequestCount;
 };
@@ -615,7 +680,6 @@ bool KFilePlacesView::isAutoResizeItemsEnabled() const
 
 void KFilePlacesView::setUrl(const QUrl &url)
 {
-    QUrl oldUrl = d->currentUrl;
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(model());
 
     if (placesModel == nullptr) {
@@ -628,23 +692,13 @@ void KFilePlacesView::setUrl(const QUrl &url)
     if (index.isValid()) {
         if (current != index && placesModel->isHidden(current) && !d->showAll) {
             KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
-            delegate->addDisappearingItem(current);
-
-            if (d->itemDisappearTimeline.state() != QTimeLine::Running) {
-                delegate->setDisappearingItemProgress(0.0);
-                d->itemDisappearTimeline.start();
-            }
+            d->addDisappearingItem(delegate, current);
         }
 
         if (current != index && placesModel->isHidden(index) && !d->showAll) {
             KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
             delegate->addAppearingItem(index);
-
-            if (d->itemAppearTimeline.state() != QTimeLine::Running) {
-                delegate->setAppearingItemProgress(0.0);
-                d->itemAppearTimeline.start();
-            }
-
+            d->triggerItemAppearingAnimation();
             setRowHidden(index.row(), false);
         }
 
@@ -684,11 +738,7 @@ void KFilePlacesView::setShowAll(bool showAll)
                 delegate->addAppearingItem(index);
             }
         }
-
-        if (d->itemAppearTimeline.state() != QTimeLine::Running) {
-            delegate->setAppearingItemProgress(0.0);
-            d->itemAppearTimeline.start();
-        }
+        d->triggerItemAppearingAnimation();
     } else {
         for (int i = 0; i < rowCount; ++i) {
             QModelIndex index = placesModel->index(i, 0);
@@ -696,11 +746,7 @@ void KFilePlacesView::setShowAll(bool showAll)
                 delegate->addDisappearingItem(index);
             }
         }
-
-        if (d->itemDisappearTimeline.state() != QTimeLine::Running) {
-            delegate->setDisappearingItemProgress(0.0);
-            d->itemDisappearTimeline.start();
-        }
+        d->triggerItemDisappearingAnimation();
     }
 }
 
@@ -715,14 +761,16 @@ void KFilePlacesView::keyPressEvent(QKeyEvent *event)
 void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(model());
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(itemDelegate());
 
-    if (placesModel == nullptr) {
+    if (!placesModel) {
         return;
     }
 
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
+
     QModelIndex index = indexAt(event->pos());
-    QString label = placesModel->text(index).replace('&', QLatin1String("&&"));
+    const QString label = placesModel->text(index).replace(QLatin1Char('&'), QLatin1String("&&"));
+    const QUrl placeUrl = placesModel->url(index);
 
     QMenu menu;
 
@@ -733,11 +781,20 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
     QAction *teardown = nullptr;
     QAction *add = nullptr;
     QAction *mainSeparator = nullptr;
+    QAction *hideSection = nullptr;
+    QAction *properties = nullptr;
+    QAction *mount = nullptr;
 
     const bool clickOverHeader = delegate->pointIsHeaderArea(event->pos());
-    if (!clickOverHeader && index.isValid()) {
+    if (clickOverHeader) {
+        const KFilePlacesModel::GroupType type = placesModel->groupType(index);
+        hideSection = menu.addAction(QIcon::fromTheme(QStringLiteral("hint")), i18n("Hide Section"));
+        hideSection->setCheckable(true);
+        hideSection->setChecked(placesModel->isGroupHidden(type));
+    }
+    else if (index.isValid()) {
         if (!placesModel->isDevice(index)) {
-            if (placesModel->url(index).toString() == QLatin1String("trash:/")) {
+            if (placeUrl.toString() == QLatin1String("trash:/")) {
                 emptyTrash = menu.addAction(QIcon::fromTheme(QStringLiteral("trash-empty")), i18nc("@action:inmenu", "Empty Trash"));
                 KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
                 emptyTrash->setEnabled(!trashConfig.group("Status").readEntry("Empty", true));
@@ -745,7 +802,6 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
             }
             add = menu.addAction(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Add Entry..."));
             mainSeparator = menu.addSeparator();
-            edit = menu.addAction(QIcon::fromTheme(QStringLiteral("document-properties")), i18n("&Edit Entry '%1'...", label));
         } else {
             eject = placesModel->ejectActionForIndex(index);
             if (eject != nullptr) {
@@ -755,28 +811,51 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 
             teardown = placesModel->teardownActionForIndex(index);
             if (teardown != nullptr) {
+                // Disable teardown option for root and home partitions
+                bool teardownEnabled = placeUrl != QUrl::fromLocalFile(QDir::rootPath());
+                if (teardownEnabled) {
+                    KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByPath(QDir::homePath());
+                    if (mountPoint && placeUrl == QUrl::fromLocalFile(mountPoint->mountPoint())) {
+                        teardownEnabled = false;
+                    }
+                }
+                teardown->setEnabled(teardownEnabled);
+
                 teardown->setParent(&menu);
                 menu.addAction(teardown);
             }
 
-            if (teardown != nullptr || eject != nullptr) {
+            if (placesModel->setupNeeded(index)) {
+                mount = menu.addAction(QIcon::fromTheme(QStringLiteral("media-mount")), i18nc("@action:inmenu", "Mount"));
+            }
+
+            if (teardown != nullptr || eject != nullptr || mount != nullptr) {
                 mainSeparator = menu.addSeparator();
             }
         }
         if (add == nullptr) {
             add = menu.addAction(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Add Entry..."));
         }
+        if (placeUrl.isLocalFile()) {
+            properties = menu.addAction(QIcon::fromTheme(QStringLiteral("document-properties")), i18n("Properties"));
+        }
+        if (!placesModel->isDevice(index)) {
+            edit = menu.addAction(QIcon::fromTheme(QStringLiteral("edit-entry")), i18n("&Edit Entry '%1'...", label));
+        }
 
-        hide = menu.addAction(i18n("&Hide Entry '%1'", label));
+        hide = menu.addAction(QIcon::fromTheme(QStringLiteral("hint")), i18n("&Hide Entry '%1'", label));
         hide->setCheckable(true);
         hide->setChecked(placesModel->isHidden(index));
+        // if a parent is hidden no interaction should be possible with children, show it first to do so
+        hide->setEnabled(!placesModel->isGroupHidden(placesModel->groupType(index)));
+
     } else {
         add = menu.addAction(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Add Entry..."));
     }
 
     QAction *showAll = nullptr;
     if (placesModel->hiddenCount() > 0) {
-        showAll = new QAction(i18n("&Show All Entries"), &menu);
+        showAll = new QAction(QIcon::fromTheme(QStringLiteral("visibility")), i18n("&Show All Entries"), &menu);
         showAll->setCheckable(true);
         showAll->setChecked(d->showAll);
         if (mainSeparator == nullptr) {
@@ -798,7 +877,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 
     QAction *result = menu.exec(event->globalPos());
 
-    if (emptyTrash != nullptr && result == emptyTrash) {
+    if (emptyTrash && (result == emptyTrash)) {
 
         KIO::JobUiDelegate uiDelegate;
         uiDelegate.setWindow(window());
@@ -807,7 +886,9 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
             KJobWidgets::setWindow(job, window());
             job->uiDelegate()->setAutoErrorHandlingEnabled(true);
         }
-    } else if (edit != nullptr && result == edit) {
+    } else if (properties && (result == properties)) {
+        KPropertiesDialog::showDialog(placeUrl, this);
+    } else if (edit && (result == edit)) {
         KBookmark bookmark = placesModel->bookmarkForIndex(index);
         QUrl url = bookmark.url();
         QString label = bookmark.text();
@@ -824,27 +905,31 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
             placesModel->editPlace(index, label, url, iconName, appName);
         }
 
-    } else if (remove != nullptr && result == remove) {
+    } else if (remove && (result == remove)) {
         placesModel->removePlace(index);
-    } else if (hide != nullptr && result == hide) {
+    } else if (hideSection && (result == hideSection)) {
+        const KFilePlacesModel::GroupType type = placesModel->groupType(index);
+        placesModel->setGroupHidden(type, hideSection->isChecked());
+
+        if (!d->showAll && hideSection->isChecked()) {
+            delegate->addDisappearingItemGroup(index);
+            d->triggerItemDisappearingAnimation();
+        }
+    } else if (hide && (result == hide)) {
         placesModel->setPlaceHidden(index, hide->isChecked());
         QModelIndex current = placesModel->closestItem(d->currentUrl);
 
         if (index != current && !d->showAll && hide->isChecked()) {
             delegate->addDisappearingItem(index);
-
-            if (d->itemDisappearTimeline.state() != QTimeLine::Running) {
-                delegate->setDisappearingItemProgress(0.0);
-                d->itemDisappearTimeline.start();
-            }
+            d->triggerItemDisappearingAnimation();
         }
-    } else if (showAll != nullptr && result == showAll) {
+    } else if (showAll && (result == showAll)) {
         setShowAll(showAll->isChecked());
-    } else if (teardown != nullptr && result == teardown) {
+    } else if (teardown && (result == teardown)) {
         placesModel->requestTeardown(index);
-    } else if (eject != nullptr && result == eject) {
+    } else if (eject && (result == eject)) {
         placesModel->requestEject(index);
-    } else if (add != nullptr && result == add) {
+    } else if (add && (result == add)) {
         QUrl url = d->currentUrl;
         QString label;
         QString iconName = QStringLiteral("folder");
@@ -858,6 +943,8 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 
             placesModel->addPlace(label, url, iconName, appName, index);
         }
+    } else if (mount && (result == mount)) {
+        placesModel->requestSetup(index);
     }
 
     index = placesModel->closestItem(d->currentUrl);
@@ -887,7 +974,7 @@ void KFilePlacesView::dragEnterEvent(QDragEnterEvent *event)
     QListView::dragEnterEvent(event);
     d->dragging = true;
 
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
     delegate->setShowHoverIndication(false);
 
     d->dropRect = QRect();
@@ -898,7 +985,7 @@ void KFilePlacesView::dragLeaveEvent(QDragLeaveEvent *event)
     QListView::dragLeaveEvent(event);
     d->dragging = false;
 
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
     delegate->setShowHoverIndication(true);
 
     setDirtyRegion(d->dropRect);
@@ -949,7 +1036,7 @@ void KFilePlacesView::dropEvent(QDropEvent *event)
     QListView::dropEvent(event);
     d->dragging = false;
 
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
     delegate->setShowHoverIndication(true);
 }
 
@@ -1023,8 +1110,10 @@ void KFilePlacesView::setModel(QAbstractItemModel *model)
     // it after the remove only).
     connect(model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
             this, SLOT(adaptItemSize()), Qt::QueuedConnection);
-    connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-            d->watcher, SLOT(currentIndexChanged(QModelIndex)));
+    connect(selectionModel(), &QItemSelectionModel::currentChanged,
+            d->watcher, &KFilePlacesEventWatcher::currentIndexChanged);
+
+    static_cast<KFilePlacesViewDelegate *>(itemDelegate())->clearFreeSpaceInfo();
 }
 
 void KFilePlacesView::rowsInserted(const QModelIndex &parent, int start, int end)
@@ -1032,22 +1121,20 @@ void KFilePlacesView::rowsInserted(const QModelIndex &parent, int start, int end
     QListView::rowsInserted(parent, start, end);
     setUrl(d->currentUrl);
 
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(itemDelegate());
-    KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(model());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(itemDelegate());
+    KFilePlacesModel *placesModel = static_cast<KFilePlacesModel *>(model());
 
     for (int i = start; i <= end; ++i) {
         QModelIndex index = placesModel->index(i, 0, parent);
         if (d->showAll || !placesModel->isHidden(index)) {
             delegate->addAppearingItem(index);
+            d->triggerItemAppearingAnimation();
         } else {
             setRowHidden(i, true);
         }
     }
 
-    if (d->itemAppearTimeline.state() != QTimeLine::Running) {
-        delegate->setAppearingItemProgress(0.0);
-        d->itemAppearTimeline.start();
-    }
+    d->triggerItemAppearingAnimation();
 
     d->adaptItemSize();
 }
@@ -1065,7 +1152,7 @@ QSize KFilePlacesView::sizeHint() const
     for (int i = 0; i < placesModel->rowCount(); ++i) {
         QModelIndex index = placesModel->index(i, 0);
         if (!placesModel->isHidden(index)) {
-            textWidth = qMax(textWidth, fm.width(index.data(Qt::DisplayRole).toString()));
+            textWidth = qMax(textWidth, fm.boundingRect(index.data(Qt::DisplayRole).toString()).width());
         }
     }
 
@@ -1073,6 +1160,15 @@ QSize KFilePlacesView::sizeHint() const
     return QSize(iconSize + textWidth + fm.height() / 2, height);
 }
 
+void KFilePlacesView::Private::addDisappearingItem(KFilePlacesViewDelegate *delegate, const QModelIndex &index)
+{
+    delegate->addDisappearingItem(index);
+    if (itemDisappearTimeline.state() != QTimeLine::Running) {
+        delegate->setDisappearingItemProgress(0.0);
+        itemDisappearTimeline.start();
+    }
+}
+
 void KFilePlacesView::Private::setCurrentIndex(const QModelIndex &index)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
@@ -1097,13 +1193,10 @@ void KFilePlacesView::Private::setCurrentIndex(const QModelIndex &index)
 
 void KFilePlacesView::Private::adaptItemSize()
 {
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
-    if (!delegate) {
-        return;
-    }
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
 
     if (!autoResizeItems) {
-        int size = q->iconSize().width(); // Assume width == height
+        const int size = q->iconSize().width(); // Assume width == height
         delegate->setIconSize(size);
         q->scheduleDelayedItemsLayout();
         return;
@@ -1140,7 +1233,7 @@ void KFilePlacesView::Private::adaptItemSize()
         QModelIndex index = placesModel->index(i, 0);
 
         if (!placesModel->isHidden(index)) {
-            textWidth = qMax(textWidth, fm.width(index.data(Qt::DisplayRole).toString()));
+            textWidth = qMax(textWidth, fm.boundingRect(index.data(Qt::DisplayRole).toString()).width());
         }
     }
 
@@ -1271,6 +1364,22 @@ int KFilePlacesView::Private::sectionsCount() const
     return count;
 }
 
+void KFilePlacesView::Private::triggerItemAppearingAnimation()
+{
+    if (itemAppearTimeline.state() != QTimeLine::Running) {
+        delegate->setAppearingItemProgress(0.0);
+        itemAppearTimeline.start();
+    }
+}
+
+void KFilePlacesView::Private::triggerItemDisappearingAnimation()
+{
+    if (itemDisappearTimeline.state() != QTimeLine::Running) {
+        delegate->setDisappearingItemProgress(0.0);
+        itemDisappearTimeline.start();
+    }
+}
+
 void KFilePlacesView::Private::_k_placeClicked(const QModelIndex &index)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
@@ -1319,8 +1428,10 @@ void KFilePlacesView::Private::_k_storageSetupDone(const QModelIndex &index, boo
 
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel *>(q->model());
 
-    QObject::disconnect(placesModel, SIGNAL(setupDone(QModelIndex,bool)),
-                        q, SLOT(_k_storageSetupDone(QModelIndex,bool)));
+    if (placesModel) {
+        QObject::disconnect(placesModel, SIGNAL(setupDone(QModelIndex,bool)),
+                            q, SLOT(_k_storageSetupDone(QModelIndex,bool)));
+    }
 
     if (success) {
         setCurrentIndex(lastClickedIndex);
@@ -1337,14 +1448,14 @@ void KFilePlacesView::Private::_k_adaptItemsUpdate(qreal value)
 
     int size = oldSize + add;
 
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
     delegate->setIconSize(size);
     q->scheduleDelayedItemsLayout();
 }
 
 void KFilePlacesView::Private::_k_itemAppearUpdate(qreal value)
 {
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
 
     delegate->setAppearingItemProgress(value);
     q->scheduleDelayedItemsLayout();
@@ -1352,7 +1463,7 @@ void KFilePlacesView::Private::_k_itemAppearUpdate(qreal value)
 
 void KFilePlacesView::Private::_k_itemDisappearUpdate(qreal value)
 {
-    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
+    KFilePlacesViewDelegate *delegate = static_cast<KFilePlacesViewDelegate *>(q->itemDelegate());
 
     delegate->setDisappearingItemProgress(value);
 
