@@ -33,6 +33,10 @@
 
 #include "kioglobal_p.h"
 
+#ifdef Q_OS_UNIX
+#include "legacycodec.h"
+#endif
+
 #include <assert.h>
 #include <errno.h>
 #ifdef Q_OS_WIN
@@ -44,11 +48,9 @@
 #endif
 
 
-#include <QByteRef>
 #include <QDate>
 #include <QVarLengthArray>
 #include <QCoreApplication>
-#include <QRegExp>
 #include <QFile>
 #include <qtemporaryfile.h>
 #ifdef Q_OS_WIN
@@ -106,6 +108,11 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char **argv)
         exit(-1);
     }
 
+#ifdef Q_OS_UNIX
+    // From Qt doc : "Note that you should not delete codecs yourself: once created they become Qt's responsibility"
+    (void)new LegacyCodec;
+#endif
+
     FileProtocol slave(argv[2], argv[3]);
 
     // Make sure the first kDebug is after the slave ctor (which sets a SIGPIPE handler)
@@ -509,24 +516,17 @@ void FileProtocol::read(KIO::filesize_t bytes)
     // qDebug() << "File::open -- read";
     Q_ASSERT(mFile && mFile->isOpen());
 
-    while (true) {
-        QByteArray res = mFile->read(bytes);
+    QVarLengthArray<char> buffer(bytes);
 
-        if (!res.isEmpty()) {
-            data(res);
-            bytes -= res.size();
-        } else {
-            // empty array designates eof
-            data(QByteArray());
-            if (!mFile->atEnd()) {
-                error(KIO::ERR_CANNOT_READ, mFile->fileName());
-                close();
-            }
-            break;
-        }
-        if (bytes <= 0) {
-            break;
-        }
+    qint64 bytesRead = mFile->read(buffer.data(), bytes);
+
+    if (bytesRead == -1) {
+        qCWarning(KIO_FILE) << "Couldn't read. Error:" << mFile->errorString();
+        error(KIO::ERR_CANNOT_READ, mFile->fileName());
+        closeWithoutFinish();
+    } else {
+        const QByteArray fileData = QByteArray::fromRawData(buffer.data(), bytesRead);
+        data(fileData);
     }
 }
 
@@ -535,17 +535,19 @@ void FileProtocol::write(const QByteArray &data)
     // qDebug() << "File::open -- write";
     Q_ASSERT(mFile && mFile->isWritable());
 
-    if (mFile->write(data) != data.size()) {
+    qint64 bytesWritten = mFile->write(data);
+
+    if (bytesWritten == -1) {
         if (mFile->error() == QFileDevice::ResourceError) { // disk full
             error(KIO::ERR_DISK_FULL, mFile->fileName());
-            close();
+            closeWithoutFinish();
         } else {
             qCWarning(KIO_FILE) << "Couldn't write. Error:" << mFile->errorString();
             error(KIO::ERR_CANNOT_WRITE, mFile->fileName());
-            close();
+            closeWithoutFinish();
         }
     } else {
-        written(data.size());
+        written(bytesWritten);
     }
 }
 
@@ -558,18 +560,22 @@ void FileProtocol::seek(KIO::filesize_t offset)
         position(offset);
     } else {
         error(KIO::ERR_CANNOT_SEEK, mFile->fileName());
-        close();
+        closeWithoutFinish();
     }
 }
 
+void FileProtocol::closeWithoutFinish()
+{
+    Q_ASSERT(mFile);
+
+    delete mFile;
+    mFile = nullptr;
+}
+
 void FileProtocol::close()
 {
     // qDebug() << "File::open -- close ";
-    Q_ASSERT(mFile);
-
-    delete mFile;
-    mFile = nullptr;
-
+    closeWithoutFinish();
     finished();
 }
 
@@ -705,6 +711,9 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                     result = -1;
                 }
             }
+        } else {
+            qCWarning(KIO_FILE) << "readData() returned" << result;
+            error(KIO::ERR_CANNOT_WRITE, dest_orig);
         }
     } while (result > 0);
 
@@ -723,8 +732,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                 }
             }
         }
-
-        ::exit(255);
+        return;
     }
 
     if (!f.isOpen()) { // we got nothing to write out, so we never opened the file
