@@ -30,6 +30,7 @@
 #include <QFile>
 #include <QDir>
 #include <qplatformdefs.h>
+#include <QStandardPaths>
 
 #include <QDebug>
 #include <kconfiggroup.h>
@@ -43,11 +44,12 @@
 #include <utime.h>
 
 #include <KAuth>
+#include <KRandom>
 
 #include "fdreceiver.h"
 
 //sendfile has different semantics in different platforms
-#if defined HAVE_SENDFILE && defined Q_OS_LINUX
+#if HAVE_SENDFILE && defined Q_OS_LINUX
 #define USE_SENDFILE 1
 #endif
 
@@ -72,7 +74,8 @@ same_inode(const QT_STATBUF &src, const QT_STATBUF &dest)
 
 static const QString socketPath()
 {
-    return QStringLiteral("org_kde_kio_file_helper_%1").arg(getpid());
+    const QString runtimeDir = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation);
+    return QStringLiteral("%1/filehelper%2%3").arg(runtimeDir, KRandom::randomString(6)).arg(getpid());
 }
 
 bool FileProtocol::privilegeOperationUnitTestMode()
@@ -84,7 +87,7 @@ bool FileProtocol::privilegeOperationUnitTestMode()
 PrivilegeOperationReturnValue FileProtocol::tryOpen(QFile &f, const QByteArray &path, int flags, int mode, int errcode)
 {
     const QString sockPath = socketPath();
-    FdReceiver fdRecv(sockPath);
+    FdReceiver fdRecv(QFile::encodeName(sockPath).toStdString());
     if (!fdRecv.isListening()) {
         return PrivilegeOperationReturnValue::failure(errcode);
     }
@@ -138,9 +141,10 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     // qDebug() << "copy(): " << srcUrl << " -> " << destUrl << ", mode=" << _mode;
 
     const QString src = srcUrl.toLocalFile();
-    const QString dest = destUrl.toLocalFile();
+    QString dest = destUrl.toLocalFile();
     QByteArray _src(QFile::encodeName(src));
     QByteArray _dest(QFile::encodeName(dest));
+    QByteArray _dest_backup;
 
     QT_STATBUF buff_src;
 #if HAVE_POSIX_ACL
@@ -177,25 +181,29 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
             return;
         }
 
-        if (!(_flags & KIO::Overwrite)) {
+        if (_flags & KIO::Overwrite) {
+            // If the destination is a symlink and overwrite is TRUE,
+            // remove the symlink first to prevent the scenario where
+            // the symlink actually points to current source!
+            if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_LNK) {
+                //qDebug() << "copy(): LINK DESTINATION";
+                if (!QFile::remove(dest)) {
+                    if (auto err = execWithElevatedPrivilege(DEL, {_dest}, errno)) {
+                        if (!err.wasCanceled()) {
+                            error(KIO::ERR_CANNOT_DELETE_ORIGINAL, dest);
+                        }
+                        return;
+                    }
+                }
+            } else if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_REG) {
+                _dest_backup = _dest;
+                dest.append(QStringLiteral(".part"));
+                _dest = QFile::encodeName(dest);
+            }
+        } else {
             error(KIO::ERR_FILE_ALREADY_EXIST, dest);
             return;
         }
-
-        // If the destination is a symlink and overwrite is TRUE,
-        // remove the symlink first to prevent the scenario where
-        // the symlink actually points to current source!
-        if ((_flags & KIO::Overwrite) && ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_LNK)) {
-            //qDebug() << "copy(): LINK DESTINATION";
-            if (!QFile::remove(dest)) {
-                if (auto err = execWithElevatedPrivilege(DEL, {_dest}, errno)) {
-                    if (!err.wasCanceled()) {
-                        error(KIO::ERR_CANNOT_DELETE_ORIGINAL, dest);
-                    }
-                    return;
-                }
-            }
-        }
     }
 
     QFile src_file(src);
@@ -248,10 +256,11 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
 
     KIO::filesize_t processed_size = 0;
     char buffer[ MAX_IPC_SIZE ];
-    int n;
+    ssize_t n = 0;
 #ifdef USE_SENDFILE
     bool use_sendfile = buff_src.st_size < 0x7FFFFFFF;
 #endif
+    bool existing_dest_delete_attempted = false;
     while (1) {
 #ifdef USE_SENDFILE
         if (use_sendfile) {
@@ -275,6 +284,11 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
             if (use_sendfile) {
                 // qDebug() << "sendfile() error:" << strerror(errno);
                 if (errno == ENOSPC) { // disk full
+                    if (!_dest_backup.isEmpty() && !existing_dest_delete_attempted) {
+                        ::unlink(_dest_backup.constData());
+                        existing_dest_delete_attempted = true;
+                        continue;
+                    }
                     error(KIO::ERR_DISK_FULL, dest);
                 } else {
                     error(KIO::ERR_SLAVE_DEFINED,
@@ -304,6 +318,11 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
 #endif
             if (dest_file.write(buffer, n) != n) {
                 if (dest_file.error() == QFileDevice::ResourceError) {  // disk full
+                    if (!_dest_backup.isEmpty() && !existing_dest_delete_attempted) {
+                        ::unlink(_dest_backup.constData());
+                        existing_dest_delete_attempted = true;
+                        continue;
+                    }
                     error(KIO::ERR_DISK_FULL, dest);
                 } else {
                     qCWarning(KIO_FILE) << "Couldn't write[2]. Error:" << dest_file.errorString();
@@ -357,9 +376,11 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
         const int errCode = errno;
         KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest);
         // Eat the error if the filesystem apparently doesn't support chmod.
+        // This test isn't fullproof though, vboxsf (VirtualBox shared folder) supports
+        // chmod if the host is Linux, and doesn't if the host is Windows. Hard to detect.
         if (mp && mp->testFileSystemFlag(KMountPoint::SupportsChmod)) {
             if (tryChangeFileAttr(CHMOD, {_dest, _mode}, errCode)) {
-                warning(i18n("Could not change permissions for '%1'", dest));
+                qCWarning(KIO_FILE) << "Could not change permissions for" << dest;
             }
         }
     }
@@ -376,7 +397,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
         (void)::chown(_dest.data(), buff_src.st_uid, -1 /*keep group*/);
     } else {
         if (tryChangeFileAttr(CHOWN, {_dest, buff_src.st_uid, buff_src.st_gid}, errno)) {
-            qCWarning(KIO_FILE) << QStringLiteral("Couldn't preserve group for '%1'").arg(dest);
+            qCWarning(KIO_FILE) << "Couldn't preserve group for" << dest;
         }
     }
 
@@ -386,7 +407,17 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     ut.modtime = buff_src.st_mtime;
     if (::utime(_dest.data(), &ut) != 0) {
         if (tryChangeFileAttr(UTIME, {_dest, qint64(ut.actime), qint64(ut.modtime)}, errno)) {
-            qCWarning(KIO_FILE) << QStringLiteral("Couldn't preserve access and modification time for '%1'").arg(dest);
+            qCWarning(KIO_FILE) << "Couldn't preserve access and modification time for" << dest;
+        }
+    }
+
+    if (!_dest_backup.isEmpty()) {
+        if (::unlink(_dest_backup.constData()) == -1) {
+            qCWarning(KIO_FILE) << "Couldn't remove original dest" <<  _dest_backup << "(" << strerror(errno) << ")";
+        }
+
+        if (::rename(_dest.constData(), _dest_backup.constData()) == -1) {
+            qCWarning(KIO_FILE) << "Couldn't rename" << _dest << "to" << _dest_backup << "(" << strerror(errno) << ")";
         }
     }
 
@@ -532,9 +563,9 @@ void FileProtocol::listDir(const QUrl &url)
          *
          */
         if (details == 0) {
-            entry.insert(KIO::UDSEntry::UDS_NAME, filename);
+            entry.fastInsert(KIO::UDSEntry::UDS_NAME, filename);
 #ifdef HAVE_DIRENT_D_TYPE
-            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
+            entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE,
                          (ep->d_type == DT_DIR) ? S_IFDIR : S_IFREG);
             const bool isSymLink = (ep->d_type == DT_LNK);
 #else
@@ -542,14 +573,14 @@ void FileProtocol::listDir(const QUrl &url)
             if (QT_LSTAT(ep->d_name, &st) == -1) {
                 continue; // how can stat fail?
             }
-            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
+            entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE,
                          ((st.st_mode & QT_STAT_MASK) == QT_STAT_DIR) ? S_IFDIR : S_IFREG);
             const bool isSymLink = ((st.st_mode & QT_STAT_MASK) == QT_STAT_LNK);
 #endif
             if (isSymLink) {
                 // for symlinks obey the UDSEntry contract and provide UDS_LINK_DEST
                 // even if we don't know the link dest (and DeleteJob doesn't care...)
-                entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("Dummy Link Target"));
+                entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("Dummy Link Target"));
             }
             listEntry(entry);
 
@@ -557,7 +588,20 @@ void FileProtocol::listDir(const QUrl &url)
             if (createUDSEntry(filename, QByteArray(ep->d_name), entry, details)) {
 #if HAVE_SYS_XATTR_H
                 if (isNtfsHidden(filename)) {
-                    entry.insert(KIO::UDSEntry::UDS_HIDDEN, 1);
+                    bool ntfsHidden = true;
+
+                    // Bug 392913: NTFS root volume is always "hidden", ignore this
+                    if (ep->d_type == DT_DIR || ep->d_type == DT_UNKNOWN || ep->d_type == DT_LNK) {
+                        const QString fullFilePath = QDir(filename).canonicalPath();
+                        auto mountPoint = KMountPoint::currentMountPoints().findByPath(fullFilePath);
+                        if (mountPoint && mountPoint->mountPoint() == fullFilePath) {
+                            ntfsHidden = false;
+                        }
+                    }
+
+                    if (ntfsHidden) {
+                        entry.fastInsert(KIO::UDSEntry::UDS_HIDDEN, 1);
+                    }
                 }
 #endif
                 listEntry(entry);
@@ -616,7 +660,7 @@ void FileProtocol::rename(const QUrl &srcUrl, const QUrl &destUrl,
         if (auto err = execWithElevatedPrivilege(RENAME, {_src, _dest}, errno)) {
             if (!err.wasCanceled()) {
                 if ((err == EACCES) || (err == EPERM)) {
-                    error(KIO::ERR_ACCESS_DENIED, dest);
+                    error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
                 } else if (err == EXDEV) {
                     error(KIO::ERR_UNSUPPORTED_ACTION, QStringLiteral("rename"));
                 } else if (err == EROFS) { // The file is on a read-only filesystem
@@ -786,11 +830,7 @@ void FileProtocol::chown(const QUrl &url, const QString &owner, const QString &g
 void FileProtocol::stat(const QUrl &url)
 {
     if (!isLocalFileSameHost(url)) {
-        QUrl redir(url);
-        redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
-        redirection(redir);
-        // qDebug() << "redirecting to " << redir;
-        finished();
+        redirect(url);
         return;
     }
 
@@ -848,10 +888,23 @@ PrivilegeOperationReturnValue FileProtocol::execWithElevatedPrivilege(ActionType
         return PrivilegeOperationReturnValue::failure(errcode);
     }
 
+    const QUrl targetUrl = QUrl::fromLocalFile(args.first().toString()); // target is always the first item.
+    const bool useParent = action != CHOWN && action != CHMOD && action != UTIME;
+    const QString targetPath = useParent ? targetUrl.adjusted(QUrl::RemoveFilename).toLocalFile() : targetUrl.toLocalFile();
+    bool userIsOwner = QFileInfo(targetPath).ownerId() == getuid();
+    if (action == RENAME) { // for rename check src and dest owner
+        QString dest = QUrl(args[1].toString()).toLocalFile();
+        userIsOwner = userIsOwner && QFileInfo(dest).ownerId() == getuid();
+    }
+    if (userIsOwner) {
+        error(KIO::ERR_PRIVILEGE_NOT_REQUIRED, targetPath);
+        return PrivilegeOperationReturnValue::canceled();
+    }
+
     QByteArray helperArgs;
     QDataStream out(&helperArgs, QIODevice::WriteOnly);
     out << action;
-    foreach (const QVariant &arg, args) {
+    for (const QVariant &arg : args) {
         out << arg;
     }
 
