@@ -425,7 +425,7 @@ bool Ftp::ftpOpenControlConnection()
     int errorCode = 0;
     QString errorMessage;
 
-    Q_FOREACH (const QString &proxyUrl, m_proxyUrls) {
+    for (const QString &proxyUrl : qAsConst(m_proxyUrls)) {
         const QUrl url(proxyUrl);
         const QString scheme(url.scheme());
 
@@ -574,7 +574,7 @@ bool Ftp::ftpLogin(bool *userChanged)
                                 , user, lastServerResponse);
             }
 
-            if (user != QStringLiteral(FTP_LOGIN)) {
+            if (user != QLatin1String(FTP_LOGIN)) {
                 info.username = user;
             }
 
@@ -1028,7 +1028,7 @@ int Ftp::ftpOpenPortDataConnection()
         data.port = m_server->serverPort();
 
         unsigned char *pData = reinterpret_cast<unsigned char *>(&data);
-        command.sprintf("PORT %d,%d,%d,%d,%d,%d", pData[3], pData[2], pData[1], pData[0], pData[5], pData[4]);
+        command = QStringLiteral("PORT %1,%2,%3,%4,%5,%6").arg(pData[3]).arg(pData[2]).arg(pData[1]).arg(pData[0]).arg(pData[5]).arg(pData[4]);
     } else if (localAddress.protocol() == QAbstractSocket::IPv6Protocol) {
         command = QStringLiteral("EPRT |2|%2|%3|").arg(localAddress.toString()).arg(m_server->serverPort());
     }
@@ -2372,12 +2372,10 @@ void Ftp::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::JobFlags
         QT_CLOSE(iCopyFile);
     }
     ftpCloseCommand();                        // must close command!
-    if (cs != statusServerError) {
-        if (iError) {
-            error(iError, sCopyFile);
-        } else {
-            finished();
-        }
+    if (cs == statusServerError && iError) {
+        error(iError, sCopyFile);
+    } else {
+        finished();
     }
 }
 
