@@ -60,7 +60,7 @@ static bool mimeTypeListContains(const QStringList &list, const KFileItem &item)
 {
     const QString itemMimeType = item.mimetype();
 
-    foreach (const QString &i, list) {
+    for (const QString &i : list) {
 
         if (i == itemMimeType || i == QLatin1String("all/all")) {
             return true;
@@ -75,12 +75,12 @@ static bool mimeTypeListContains(const QStringList &list, const KFileItem &item)
             return true;
         }
 
-        const int iSlashPos = i.indexOf('/');
-        Q_ASSERT(i > nullptr);
+        const int iSlashPos = i.indexOf(QLatin1Char(QLatin1Char('/')));
+        Q_ASSERT(iSlashPos > 0);
         const QStringRef iSubType = i.midRef(iSlashPos+1);
 
-        if (iSubType == "*") {
-            const int itemSlashPos = itemMimeType.indexOf('/');
+        if (iSubType == QLatin1String("*")) {
+            const int itemSlashPos = itemMimeType.indexOf(QLatin1Char('/'));
             Q_ASSERT(itemSlashPos > 0);
             const QStringRef iTopLevelType = i.midRef(0, iSlashPos);
             const QStringRef itemTopLevelType = itemMimeType.midRef(0, itemSlashPos);
@@ -140,10 +140,10 @@ KFileItemActionsPrivate::KFileItemActionsPrivate(KFileItemActions *qq)
       m_parentWidget(nullptr),
       m_config(QStringLiteral("kservicemenurc"), KConfig::NoGlobals)
 {
-    QObject::connect(&m_executeServiceActionGroup, SIGNAL(triggered(QAction*)),
-                     this, SLOT(slotExecuteService(QAction*)));
-    QObject::connect(&m_runApplicationActionGroup, SIGNAL(triggered(QAction*)),
-                     this, SLOT(slotRunApplication(QAction*)));
+    QObject::connect(&m_executeServiceActionGroup, &QActionGroup::triggered,
+                     this, &KFileItemActionsPrivate::slotExecuteService);
+    QObject::connect(&m_runApplicationActionGroup, &QActionGroup::triggered,
+                     this, &KFileItemActionsPrivate::slotRunApplication);
 }
 
 KFileItemActionsPrivate::~KFileItemActionsPrivate()
@@ -191,7 +191,7 @@ int KFileItemActionsPrivate::insertServices(const ServiceList &list,
             QAction *act = new QAction(q);
             act->setObjectName(QStringLiteral("menuaction")); // for the unittest
             QString text = (*it).text();
-            text.replace('&', QLatin1String("&&"));
+            text.replace(QLatin1Char('&'), QLatin1String("&&"));
             act->setText(text);
             if (!(*it).icon().isEmpty()) {
                 act->setIcon(QIcon::fromTheme((*it).icon()));
@@ -278,7 +278,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
     // first check the .directory if this is a directory
     if (d->m_props.isDirectory() && isSingleLocal) {
-        QString dotDirectoryFile = QUrl::fromLocalFile(firstItem.localPath()).path().append("/.directory");
+        QString dotDirectoryFile = QUrl::fromLocalFile(firstItem.localPath()).path().append(QLatin1String("/.directory"));
         if (QFile::exists(dotDirectoryFile)) {
             const KDesktopFile desktopFile(dotDirectoryFile);
             const KConfigGroup cfg = desktopFile.desktopGroup();
@@ -294,10 +294,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
     const KConfigGroup showGroup = d->m_config.group("Show");
 
-    const QString commonMimeType = d->m_props.mimeType();
-    const QString commonMimeGroup = d->m_props.mimeGroup();
     const QMimeDatabase db;
-    const QMimeType mimeTypePtr = commonMimeType.isEmpty() ? QMimeType() : db.mimeTypeForName(commonMimeType);
     const KService::List entries = KServiceTypeTrader::self()->query(QStringLiteral("KonqPopupMenu/Plugin"));
     KService::List::const_iterator eEnd = entries.end();
     for (KService::List::const_iterator it2 = entries.begin(); it2 != eEnd; ++it2) {
@@ -317,7 +314,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
         }
         if (cfg.hasKey("X-KDE-ShowIfDBusCall")) {
             QString calldata = cfg.readEntry("X-KDE-ShowIfDBusCall");
-            const QStringList parts = calldata.split(' ');
+            const QStringList parts = calldata.split(QLatin1Char(' '));
             const QString &app = parts.at(0);
             const QString &obj = parts.at(1);
             QString interface = parts.at(2);
@@ -340,8 +337,8 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
         }
         if (cfg.hasKey("X-KDE-Protocol")) {
             const QString theProtocol = cfg.readEntry("X-KDE-Protocol");
-            if (theProtocol.startsWith('!')) {
-                const QString excludedProtocol = theProtocol.mid(1);
+            if (theProtocol.startsWith(QLatin1Char('!'))) {
+                const QStringRef excludedProtocol = theProtocol.midRef(1);
                 if (excludedProtocol == protocol) {
                     continue;
                 }
@@ -362,7 +359,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
         if (cfg.hasKey("X-KDE-Require")) {
             const QStringList capabilities = cfg.readEntry("X-KDE-Require", QStringList());
-            if (capabilities.contains(QStringLiteral("Write")) && !d->m_props.supportsWriting()) {
+            if (capabilities.contains(QLatin1String("Write")) && !d->m_props.supportsWriting()) {
                 continue;
             }
         }
@@ -403,9 +400,8 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
             const bool ok = std::all_of(items.constBegin(),
                                         items.constEnd(),
-                                        [&types, &excludeTypes, this](const KFileItem &i)
+                                        [&types, &excludeTypes](const KFileItem &i)
             {
-                const QString mimetype = i.mimetype();
                 return mimeTypeListContains(types, i) && !mimeTypeListContains(excludeTypes, i);
             });
 
@@ -416,7 +412,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
                 ServiceList &list = s.selectList(priority, submenuName);
                 const ServiceList userServices = KDesktopFileActions::userDefinedServices(*(*it2), isLocal, urlList);
-                foreach (const KServiceAction &action, userServices) {
+                for (const KServiceAction &action : userServices) {
                     if (showGroup.readEntry(action.name(), true)) {
                         list += action;
                     }
@@ -431,6 +427,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
             s.userPriority.count() + s.userPrioritySubmenus.count() > 1) {
         // we have more than one item, so let's make a submenu
         actionMenu = new QMenu(i18nc("@title:menu", "&Actions"), mainMenu);
+        actionMenu->setIcon(QIcon::fromTheme(QStringLiteral("view-more-symbolic")));
         actionMenu->menuAction()->setObjectName(QStringLiteral("actions_submenu")); // for the unittest
         mainMenu->addMenu(actionMenu);
     }
@@ -443,7 +440,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
             (s.user.count() > 0 ||
              s.userSubmenus.count() > 0 ||
              s.builtin.count() > 0) &&
-            !actionMenu->actions().last()->isSeparator()) {
+            !actionMenu->actions().constLast()->isSeparator()) {
         actionMenu->addSeparator();
     }
     userItemCount += d->insertServicesSubmenus(s.userSubmenus, actionMenu, false);
@@ -457,10 +454,12 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 
 int KFileItemActions::addPluginActionsTo(QMenu *mainMenu)
 {
+    QString commonMimeType = d->m_props.mimeType();
+    if (commonMimeType.isEmpty() && d->m_props.isFile()) {
+        commonMimeType = QStringLiteral("application/octet-stream");
+    }
+
     QStringList addedPlugins;
-    const QString commonMimeType = d->m_props.mimeType();
-    const QString commonMimeGroup = d->m_props.mimeGroup();
-    const QMimeDatabase db;
     int itemCount = 0;
 
     const KConfigGroup showGroup = d->m_config.group("Show");
@@ -480,8 +479,10 @@ int KFileItemActions::addPluginActionsTo(QMenu *mainMenu)
             addedPlugins.append(service->desktopEntryName());
         }
     }
+
+    const QMimeDatabase db;
     const auto jsonPlugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kfileitemaction"), [&db, commonMimeType](const KPluginMetaData& metaData) {
-        if (!metaData.serviceTypes().contains(QStringLiteral("KFileItemAction/Plugin"))) {
+        if (!metaData.serviceTypes().contains(QLatin1String("KFileItemAction/Plugin"))) {
             return false;
         }
 
@@ -539,6 +540,8 @@ KService::List KFileItemActions::associatedApplications(const QStringList &mimeT
     // Second, it ranks them based on their preference level in the associated applications list.
     // The more often a service appear near the front of the list, the LOWER its score.
 
+    rankings.reserve(firstOffers.count());
+    serviceList.reserve(firstOffers.count());
     for (int i = 0; i < firstOffers.count(); ++i) {
         KFileItemActionsPrivate::ServiceRank tempRank;
         tempRank.service = firstOffers[i];
@@ -550,6 +553,7 @@ KService::List KFileItemActions::associatedApplications(const QStringList &mimeT
     for (int j = 1; j < mimeTypeList.count(); ++j) {
         QStringList subservice; // list of services that support this mimetype
         const KService::List offers = KMimeTypeTrader::self()->query(mimeTypeList[j], QStringLiteral("Application"), traderConstraint);
+        subservice.reserve(offers.count());
         for (int i = 0; i != offers.count(); ++i) {
             const QString serviceId = offers[i]->storageId();
             subservice << serviceId;
@@ -576,7 +580,8 @@ KService::List KFileItemActions::associatedApplications(const QStringList &mimeT
     std::sort(rankings.begin(), rankings.end(), KFileItemActionsPrivate::lessRank);
 
     KService::List result;
-    Q_FOREACH (const KFileItemActionsPrivate::ServiceRank &tempRank, rankings) {
+    result.reserve(rankings.size());
+    for (const KFileItemActionsPrivate::ServiceRank &tempRank : qAsConst(rankings)) {
         result << tempRank.service;
     }
 
@@ -607,12 +612,8 @@ void KFileItemActions::addOpenWithActionsTo(QMenu *topMenu, const QString &trade
     // "Open With..." for folders is really not very useful, especially for remote folders.
     // (media:/something, or trash:/, or ftp://...)
     if (!d->m_props.isDirectory() || isLocal) {
-        if (!topMenu->actions().isEmpty()) {
-            topMenu->addSeparator();
-        }
 
         QAction *runAct = new QAction(this);
-        QString runActionName;
 
         const QStringList serviceIdList = d->listPreferredServiceIds(d->m_mimeTypeList, traderConstraint);
         //qDebug() << "serviceIdList=" << serviceIdList;
@@ -625,7 +626,7 @@ void KFileItemActions::addOpenWithActionsTo(QMenu *topMenu, const QString &trade
 
             if (serviceIdList.count() == 1) {
                 const KService::Ptr app = preferredService(d->m_mimeTypeList.first(), traderConstraint);
-                runActionName = i18n("&Open with %1", app->name());
+                runAct->setText(i18n("&Open with %1", app->name()));
                 runAct->setIcon(QIcon::fromTheme(app->icon()));
 
                 // Remove that app from the offers list (#242731)
@@ -636,27 +637,29 @@ void KFileItemActions::addOpenWithActionsTo(QMenu *topMenu, const QString &trade
                     }
                 }
             } else {
-                runActionName = i18n("&Open");
+                runAct->setText(i18n("&Open"));
             }
 
-            runAct->setText(runActionName);
-
             d->m_traderConstraint = traderConstraint;
             d->m_fileOpenList = d->m_props.items();
-            QObject::connect(runAct, SIGNAL(triggered()), d, SLOT(slotRunPreferredApplications()));
+            QObject::connect(runAct, &QAction::triggered,
+                             d, &KFileItemActionsPrivate::slotRunPreferredApplications);
             topMenu->addAction(runAct);
         }
 
         if (!offers.isEmpty()) {
             QMenu *menu = topMenu;
 
-            // Always show the top app inline
-            QAction *act = d->createAppAction(offers.takeFirst(), true);
-            menu->addAction(act);
+            // Show the top app inline for files, but not folders
+            if (!d->m_props.isDirectory()) {
+                QAction *act = d->createAppAction(offers.takeFirst(), true);
+                menu->addAction(act);
+            }
 
             // If there are still more apps, show them in a sub-menu
             if (!offers.isEmpty()) { // submenu 'open with'
                 menu = new QMenu(i18nc("@title:menu", "&Open With"), topMenu);
+                menu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
                 menu->menuAction()->setObjectName(QStringLiteral("openWith_submenu")); // for the unittest
                 topMenu->addMenu(menu);
 
@@ -674,21 +677,24 @@ void KFileItemActions::addOpenWithActionsTo(QMenu *topMenu, const QString &trade
             QString openWithActionName;
             if (menu != topMenu) { // submenu
                 menu->addSeparator();
-                openWithActionName = i18nc("@action:inmenu Open With", "&Other...");
+                openWithActionName = i18nc("@action:inmenu Open With", "&Other Application...");
             } else {
                 openWithActionName = i18nc("@title:menu", "&Open With...");
             }
             QAction *openWithAct = new QAction(this);
             openWithAct->setText(openWithActionName);
             openWithAct->setObjectName(QStringLiteral("openwith_browse")); // for the unittest
-            QObject::connect(openWithAct, SIGNAL(triggered()), d, SLOT(slotOpenWithDialog()));
+            QObject::connect(openWithAct, &QAction::triggered,
+                             d, &KFileItemActionsPrivate::slotOpenWithDialog);
             menu->addAction(openWithAct);
             menu->addSeparator();
         } else { // no app offers -> Open With...
             QAction *act = new QAction(this);
             act->setText(i18nc("@title:menu", "&Open With..."));
+            act->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
             act->setObjectName(QStringLiteral("openwith")); // for the unittest
-            QObject::connect(act, SIGNAL(triggered()), d, SLOT(slotOpenWithDialog()));
+            QObject::connect(act, &QAction::triggered,
+                             d, &KFileItemActionsPrivate::slotOpenWithDialog);
             topMenu->addAction(act);
         }
 
@@ -702,9 +708,9 @@ void KFileItemActionsPrivate::slotRunPreferredApplications()
     const QStringList mimeTypeList = listMimeTypes(fileItems);
     const QStringList serviceIdList = listPreferredServiceIds(mimeTypeList, m_traderConstraint);
 
-    foreach (const QString& serviceId, serviceIdList) {
+    for (const QString& serviceId : serviceIdList) {
         KFileItemList serviceItems;
-        foreach (const KFileItem &item, fileItems) {
+        for (const KFileItem &item : fileItems) {
             const KService::Ptr serv = preferredService(item.mimetype(), m_traderConstraint);
             const QString preferredServiceId = serv ? serv->storageId() : QString();
             if (preferredServiceId == serviceId) {
@@ -722,7 +728,7 @@ void KFileItemActionsPrivate::slotRunPreferredApplications()
             KRun::displayOpenWithDialog(serviceItems.urlList(), m_parentWidget);
             continue;
         }
-        KRun::runService(*servicePtr, serviceItems.urlList(), m_parentWidget);
+        KRun::runApplication(*servicePtr, serviceItems.urlList(), m_parentWidget);
     }
 }
 
@@ -736,9 +742,9 @@ void KFileItemActions::runPreferredApplications(const KFileItemList &fileOpenLis
 void KFileItemActionsPrivate::openWithByMime(const KFileItemList &fileItems)
 {
     const QStringList mimeTypeList = listMimeTypes(fileItems);
-    foreach (const QString& mimeType, mimeTypeList) {
+    for (const QString& mimeType : mimeTypeList) {
         KFileItemList mimeItems;
-        foreach (const KFileItem &item, fileItems) {
+        for (const KFileItem &item : fileItems) {
             if (item.mimetype() == mimeType) {
                 mimeItems << item;
             }
@@ -753,7 +759,7 @@ void KFileItemActionsPrivate::slotRunApplication(QAction *act)
     KService::Ptr app = act->data().value<KService::Ptr>();
     Q_ASSERT(app);
     if (app) {
-        KRun::runService(*app, m_props.urlList(), m_parentWidget);
+        KRun::runApplication(*app, m_props.urlList(), m_parentWidget);
     }
 }
 
@@ -767,7 +773,7 @@ void KFileItemActionsPrivate::slotOpenWithDialog()
 QStringList KFileItemActionsPrivate::listMimeTypes(const KFileItemList &items)
 {
     QStringList mimeTypeList;
-    foreach (const KFileItem &item, items) {
+    for (const KFileItem &item : items) {
         if (!mimeTypeList.contains(item.mimetype())) {
             mimeTypeList << item.mimetype();
         }
@@ -778,7 +784,8 @@ QStringList KFileItemActionsPrivate::listMimeTypes(const KFileItemList &items)
 QStringList KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &mimeTypeList, const QString &traderConstraint)
 {
     QStringList serviceIdList;
-    Q_FOREACH (const QString &mimeType, mimeTypeList) {
+    serviceIdList.reserve(mimeTypeList.size());
+    for (const QString &mimeType : mimeTypeList) {
         const KService::Ptr serv = preferredService(mimeType, traderConstraint);
         const QString newOffer = serv ? serv->storageId() : QString();
         serviceIdList << newOffer;
@@ -789,7 +796,7 @@ QStringList KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &
 
 QAction *KFileItemActionsPrivate::createAppAction(const KService::Ptr &service, bool singleOffer)
 {
-    QString actionName(service->name().replace('&', QLatin1String("&&")));
+    QString actionName(service->name().replace(QLatin1Char('&'), QLatin1String("&&")));
     if (singleOffer) {
         actionName = i18n("Open &with %1", actionName);
     } else {
