@@ -315,13 +315,7 @@ KFileItem KFileItemDelegate::Private::fileItem(const QModelIndex &index) const
 QString KFileItemDelegate::Private::replaceNewlines(const QString &text) const
 {
     QString string = text;
-    const QChar newline = QLatin1Char('\n');
-
-    for (int i = 0; i < string.length(); i++)
-        if (string[i] == newline) {
-            string[i] = QChar::LineSeparator;
-        }
-
+    string.replace(QLatin1Char('\n'), QChar::LineSeparator);
     return string;
 }
 
@@ -1578,7 +1572,7 @@ bool KFileItemDelegate::eventFilter(QObject *object, QEvent *event)
         case Qt::Key_Enter:
         case Qt::Key_Return: {
             const QString text = editor->toPlainText();
-            if (text.isEmpty() || (text == QLatin1String(".")) || (text == QLatin1String(".."))) {
+            if (text.isEmpty() || (text == QLatin1Char('.')) || (text == QLatin1String(".."))) {
                 return true;    // So a newline doesn't get inserted
             }
 
